#ifndef __cell_h_
#define __cell_h_
#include "adevs.h"
#include <fstream>

/// Indexes into initial condition data
#define LEFT 0
#define RIGHT 1
#define UP 2
#define DOWN 3
#define CENTER 4

/**
A cell used in the finite difference solution of the heat equation.
*/
class cell: public atomic, public cell_interface
{
	public:
		/// Quantum size
		static double D;
		/// Grid resolution in the x and y dimensions
		static double dx, dy;
		/// Diffusion rate
		static double k;
		/// State transition count
		static unsigned long delta_s_count;
		/// Smallest exhibited time advance
		static double ta_min;
		/// Dimensions of the cellspace
		static int width, height;
		
		cell();
		cell(const cell& src);
		const cell& operator=(const cell& src);
		/// Initialization function
		void init();
		/// Internal transition function
		void delta_int();
		/// External transition function
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		/// Confluent transition function
		void delta_conf(const adevs_bag<PortValue>& x);
		/// Output function
		void output_func(adevs_bag<PortValue>& _y);
		void gc_output(adevs_bag<PortValue>& g);
		void report(std::ofstream& stream) const;
		object* clone() const;
		~cell();
		void set_init_conditions(double u0[5]);
		void set_pos(int x, int y);
	private:	
		/// Last known values of neighboring cells
		double u[4];
		/// Quantized integrator
		qoden<1> q;
		double du;
		/// Position in the cell space
		int x, y;
		/// Compute derivatives at cell and handle initial conditions
		double compute_derivative();
		double compute_derivative(double center_val);

};

#endif
