#include "cellBlocks.h"
using namespace std;

cellBlocks::net_exec::net_exec(devs* parent, int width, int height, int num_partitions):
netExec(parent),
w(width),
h(height)
{
	int i, j;
	int block_width = w;
	int block_width_extra = 0;
	int block_height = h;
	int block_height_extra = 0;
	num_xblocks = 1;
	num_yblocks = 1;
	for (i = 1; i < num_partitions; i++)
	{
		if (block_width < block_height)
		{
			num_yblocks++;
			block_height = div(h,num_yblocks).quot;
			block_height_extra = div(h,num_yblocks).rem;
		}
		else
		{
			num_xblocks++;
			block_width = div(w,num_xblocks).quot;
			block_width_extra = div(w,num_yblocks).rem;
		}
	}

	space = new cellSpace**[num_xblocks];
	for (i = 0; i < num_xblocks; i++)
	{
		space[i] = new cellSpace*[num_yblocks];
	}
	for (i = 0; i < num_xblocks; i++)
	{
		for (j = 0; j < num_yblocks; j++)
		{
			int extra = (i == num_xblocks-1 && j == num_yblocks-1);
			int bw = block_width + extra*block_width_extra;
			int bh = block_height + extra*block_height_extra;
			space[i][j] = new cellSpace(cellSpace::SIX_POINT,bw,bh);
			cellSpace::coord_t pos(i,j);
			coord_table[space[i][j]] = pos;
		}
	}
}	

void cellBlocks::net_exec::init()
{
	passivate();
}

void cellBlocks::net_exec::getComponents(adevs_set<devs*>& c)
{
	for (int i = 0; i < num_xblocks; i++)
	{
		for (int j = 0; j < num_yblocks; j++)
		{
			c.add(space[i][j]);
		}
	}
}

void cellBlocks::net_exec::addModel(devs* d, int x, int y)
{
	int block_x = div(x,div(w,num_xblocks).quot).quot;
	int block_y = div(y,div(h,num_yblocks).quot).quot;
	if (block_x >= num_xblocks) block_x = num_xblocks - 1;
	if (block_y >= num_yblocks) block_y = num_yblocks - 1;
	int cell_x = x-block_x*space[0][0]->getWidth();
	int cell_y = y-block_y*space[0][0]->getHeight();
	space[block_x][block_y]->add(d,cell_x,cell_y);
}

const devs* cellBlocks::net_exec::getModel(int x, int y) const
{
	int block_x = div(x,div(w,num_xblocks).quot).quot;
	int block_y = div(y,div(h,num_yblocks).quot).quot;
	if (block_x >= num_xblocks) block_x = num_xblocks - 1;
	if (block_y >= num_yblocks) block_y = num_yblocks - 1;
	int cell_x = x-block_x*space[0][0]->getWidth();
	int cell_y = y-block_y*space[0][0]->getHeight();
	return space[block_x][block_y]->getModel(cell_x,cell_y);
}

void cellBlocks::net_exec::route(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	/// Get the location inside the partition from which the event originated
	cellSpace::coord_t pos(coord_table[(cellSpace*)model]);
	cellSpace* partition = space[pos.x][pos.y];
	cellSpace::coord_t event_origin(partition->getCoordinates(pv.port));
	/// Send left
	if (event_origin.x == 0 && pos.x > 0)
	{
		EventReceiver e;
		cellSpace* block = space[pos.x-1][pos.y];
		e.model = block;
		e.port = block->getPort(block->getWidth()-1,event_origin.y);
		r.add(e);
	}
	/// Send right
	if (event_origin.x == partition->getWidth()-1 && pos.x < num_xblocks-1)
	{
		EventReceiver e;
		cellSpace* block = space[pos.x+1][pos.y];
		e.model = block;
		e.port = block->getPort(0,event_origin.y);
		r.add(e);
	}
	/// Send up
	if (event_origin.y == 0 && pos.y > 0)
	{
		EventReceiver e;
		cellSpace* block = space[pos.x][pos.y-1];
		e.model = block;
		e.port = block->getPort(event_origin.x,block->getHeight()-1);
		r.add(e);
	}
	/// Send down
	if (event_origin.y == partition->getHeight()-1 && pos.y < num_yblocks-1)
	{
		EventReceiver e;
		cellSpace* block = space[pos.x][pos.y+1];
		e.model = block;
		e.port = block->getPort(event_origin.x,0);
		r.add(e);
	}
}

cellBlocks::net_exec::~net_exec()
{
	for (int i = 0; i < num_xblocks; i++)
	{
		for (int j = 0; j < num_yblocks; j++)
		{
			delete space[i][j];
		}
		delete [] space[i];
	}
	delete [] space;
}
