#include "cell.h"

const port_t cell::flux_left = 0;
const port_t cell::flux_right = 1;
const port_t cell::soln = 2;
double cell::dx = 1.0;
double cell::T_min = INFINITY;
double cell::D = 0.0;
		
cell::cell():
atomic(),
index(-1)
{}

cell::cell(int x, double u0):
atomic(),
c(0),
index(x),
dq(0.0)
{
	q.setParams(D);
	q.init(0,u0);
}

cell::cell(const cell& src):
atomic(src),
q(src.q),
fl(src.fl),
fr(src.fr),
c(src.c),
index(src.index),
dq(src.dq)
{}

const cell& cell::operator=(const cell& src)
{
	atomic::operator=(src);
	q = src.q;
	fl = src.fl;
	fr = src.fr;
	c = src.c;
	index = src.index;
	dq = src.dq;
	return *this;
}

void cell::init()
{
	c = 0;
	T_last = 0.0;
	hold(q.ta());
}

void cell::delta_int()
{
	c++;
	if (timeCurrent()-T_last > 0.0)
		T_min = min(timeCurrent()-T_last,T_min);
	T_last = timeCurrent();
	q.integ(q.ta());
	hold(q.ta());
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	c++;
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == flux_left)
			fl = x.get(i).value.get_value();
		else
			fr = x.get(i).value.get_value();
	}
	dq = -(fl+fr)/dx;
	q.integ(e,&dq);
	hold(q.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& x)
{
	c++;
	if (timeCurrent()-T_last > 0.0)
		T_min = min(timeCurrent()-T_last,T_min);
	T_last = timeCurrent();
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == flux_left)
			fl = x.get(i).value.get_value();
		else
			fr = x.get(i).value.get_value();
	}
	dq = -(fl+fr)/dx;
	q.integ(q.ta(),&dq);
	hold(q.ta());
}

void cell::output_func(adevs_bag<PortValue>& y)
{
	IOtype o(IOtype::CELL,index,q.out());
	output(soln,o,y);
}

double cell::getDerivative() const
{
	return dq;
}

double cell::getState() const
{
	return q.state();
}
