#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>
#include <list>

/**
A queueable entity.  Has a size which indicates
the time required to process the entity.
*/
class entity: public object 
{
	public:
		entity(double size = 0.0):
		object(),mSize(size){} 
		entity(const entity& src):
		object(src),mSize(src.mSize){}
		const entity& operator=(const entity& src) 
		{
			mSize = src.mSize;
			return *this;
		}
		object* clone() const 
		{
			return new entity(*this); 
		}
		double get_size() const { return mSize; }
		void set_size(double size) { mSize = size; }
		bool operator<(const entity& x) const 
		{
			return (mSize < x.mSize);
		}
		bool operator<=(const entity& x) const 
		{
			return (mSize <= x.mSize);
		}
		bool operator>(const entity& x) const 
		{
			return (mSize > x.mSize);
		}
		bool operator>=(const entity& x) const 
		{
			return (mSize >= x.mSize);
		}
		bool operator==(const entity& x) const 
		{
			return (mSize == x.mSize);
		}
		bool operator!=(const entity& x) const 
		{
			return (mSize != x.mSize);
		}
	private:
		double mSize;
};

class cell_data: public object 
{
	public:
		cell_data(int ID, int queue_size):
		object(),mID(ID),mSize(queue_size) {}
		cell_data(const cell_data& src):
		object(src),mID(src.mID),mSize(src.mSize) {}
		object* clone() const { return new cell_data(*this); }
		int getID() const { return mID; }
		int getSize() const { return mSize; }
	private:
		int mID;
		int mSize;
};

/**
A single FIFO queue that accepts entity
objects.  Objects in queue receive processing
time simultaneously, and so multiple objects
can exit at a single point in time.
*/
class cell: public atomic 
{
	public:
		static const port_t in;
		static const port_t out;
		static const port_t stats;
		cell():atomic(){}
		cell(int ID):atomic(),ID(ID){}
		cell(const cell& src):atomic(src),q(src.q),ID(src.ID){}
		const cell& operator=(const cell& src)
		{
			atomic::operator=(src);
			q = src.q;
			ID = src.ID;
			return *this;
		}
		object* clone() const { return new cell(*this); }
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g);
		~cell(){}
	private:
		std::list<std::pair<double,entity> > q;
		int ID;
};

#endif
