#ifndef __adevs_config_h_
#define __adevs_config_h_
#include <limits.h>
#include <float.h>
#include <math.h>

// All configurable types need to have a default (no arg) constructor,
// assignment operator, and copy constructor.

/**
Define the type for model inputs and outputs.
*/
typedef enum { BURNING, BURNT, UNBURNT } CellPhase;
#define ADEVS_IO_TYPE CellPhase

/**
Define the type used as model ports.  The port_t type
can always be used as an alias for ports.  The port
type must support the equality (== and !=) operators.
*/
#define ADEVS_PORT_TYPE int

/**
Define the simulation clock type.  EPSILON is used as a time granule
for determined when a time t is 'equal' to a time t +- EPSILON.
The stime_t type can always be used as an alias for time.
Time should support operators sufficient to make it an Abelian
group using the + and - operators.  It should also
support ALL of the comparison (== != <= >= > <) operators
interpreted in the usual way.
*/ 
#define ADEVS_TIME_TYPE unsigned
#define ADEVS_ZERO 0
#define ADEVS_EPSILON 0
#define ADEVS_INFINITY UINT_MAX

#endif
